@@ -2,8 +2,8 @@
 # NEXUS.pm
 ######################################################
 #
-# $Id: NEXUS.pm,v 1.118 2010/09/22 20:09:37 astoltzfus Exp $
-# $Revision: 1.118 $
+# $Id: NEXUS.pm,v 1.121 2012/02/08 15:14:34 astoltzfus Exp $
+# $Revision: 1.121 $
 #
 #################### START POD DOCUMENTATION ##################
 
@@ -59,6 +59,9 @@ use Bio::NEXUS::UnalignedBlock;
 use Bio::NEXUS::UnknownBlock;
 use Bio::NEXUS::DataBlock;
 use Bio::NEXUS::DistancesBlock;
+# in progress
+use Bio::NEXUS::CodonsBlock; 
+use Bio::NEXUS::NotesBlock; 
 
 use Bio::NEXUS::Util::Logger;
 use Bio::NEXUS::Util::Exceptions 'throw';
@@ -70,7 +73,7 @@ use Bio::NEXUS::Util::Exceptions 'throw';
 # ExtUtils::MakeMaker reads package global $VERSION
 
 use vars qw($VERSION $AUTOLOAD);
-$VERSION = do { my @r = ( q$Name:  $ =~ /\d+/g ); ( $#r < 0 ) ? '0.76' : sprintf " %d." . "%02d" x $#r, @r; };
+$VERSION = do { my @r = ( q$Name:  $ =~ /\d+/g ); ( $#r < 0 ) ? '0.77' : sprintf " %d." . "%02d" x $#r, @r; };
 
 # a logger is an object that conditionally prints messages,
 # so we don't need to add print statements and then comment
@@ -365,12 +368,12 @@ sub create_block {
     my %block_types = (
         'assumptions' => "Bio::NEXUS::AssumptionsBlock",
         'characters'  => "Bio::NEXUS::CharactersBlock",
-
+# in progress: codons
         # 'codons'      =>    "Bio::NEXUS::CodonsBlock",
         'data'      => "Bio::NEXUS::DataBlock",
         'distances' => "Bio::NEXUS::DistancesBlock",
         'history'   => "Bio::NEXUS::HistoryBlock",
-
+# in progress: notes
         # 'notes'       =>    "Bio::NEXUS::NotesBlock",
         'sets'      => "Bio::NEXUS::SetsBlock",
         'span'      => "Bio::NEXUS::SpanBlock",
